function edf_downsample(varargin)
% EDF_DOWNSAMPLE.M
% Downsamples (bins) a (series of) EDF files.
%
% Usage:
% edf_downsample(filename)
%   will downsample the EDF file (with or without extensions) specified with 
%   binning of 2 and write to filename_binned.edf 
% 
% Greg Johnson, May 2006
%

fnamein=varargin{1};
fparts=utilExtractFilenameParts(fnamein);

if isempty(fparts.extension) % user didn't supply filename with extension
  fparts.extension='edf';
end

fnameout=[fparts.prefix 'binned_' fparts.index '.' fparts.extension];
fnamein=[fparts.prefix fparts.index '.' fparts.extension];
% check file exists
if ~exist(fnamein)
  disp('Can''t see file - sorry!')
  return
end

% read original file
im=edf_read(fnamein);

% bin 2x2
sizeim=size(im);
if mod(sizeim(1),2)==1  % odd number of rows - crop from end if so
  im=im(1:end-1,:);
end
if mod(sizeim(2),2)==1 % odd number of columesn - crop from end if so
  im=im(:,1:end-1);
end
      
im_d=(im(1:2:end,1:2:end)+im(1:2:end,2:2:end)+im(2:2:end,1:2:end)+im(2:2:end,2:2:end))/4;

% write result.  May not be the best choice to use floats, but it is flexible...
edf_write(im_d,fnameout,'float32')


 


    


if 0
% THIS CODE IS FOR SEQUENCES, BUT SHOULD BE HANDLED BETTER
if nargin==0
  disp('Will try to downsample 1500 files')
  a=0:10:1490;
  a=reshape(a,1,1,150);
  b=a+9;
  parallel('edf_downsample',a,b)
elseif nargin==2
  first=varargin{1};
  last=varargin{2};

  for n=first:last
    fprintf('Slice: %d/%d...',n,last);
    fnamein=sprintf('foil200_nc2_flat_%04d.edf',n);
    fnameout=sprintf('small_foil200_nc2_flat_%04d.edf',n);
    if ~exist(fnameout,'file')
      im=edfread(fnamein);
      sizeim=size(im);
      if mod(sizeim(1),2)==1
        im=im(1:end-1,:);
      end
      if mod(sizeim(2),2)==1
        im=im(:,1:end-1);
      end
      
      im_d=(im(1:2:end,1:2:end)+im(1:2:end,2:2:end)+im(2:2:end,1:2:end)+im(2:2:end,2:2:end))/4;
      edfwrite(fnameout,im_d,'float32')
    else
      fprintf('skipping');
    end
    fprintf('\r');
  end
  fprintf('\n')

elseif nargin==1
  disp('Trying to downsample file specified')
  im=edfread(varargin{1});
  im_d=(im(1:2:end,1:2:end)+im(1:2:end,2:2:end)+im(2:2:end,1:2:end)+im(2:2:end,2:2:end))/4;
  edfwrite(fnameout,im_d,'float32')

else
    disp('Not sure what you want!')
end
end % if 0

end
