function edf_write(varargin)
% EDF_WRITE.M An EDF writer than handles images and 3D volumes
% Usage:
% edfwrite(im,filename)
% the datatype of the EDF file will be the same as the Matlab variable
% (usually double - 64 bits)
% edfwrite(vol,filename,'uint16')
% the datatype will be uint16 instead.  See matlab help on DATATYPES
%
% WARNING!  THESE IMAGES DO NOT NEED TRANSPOSING BEFORE WRITING!
% Designed to be used from imwrite (see edf_setup_imread.m 
% for how this is done automatically).
% Greg Johnson
% June 2006

if nargin==1
  data=varargin{1};
elseif nargin==2
  data=varargin{1};
  filename=varargin{2};
elseif nargin==3
  whocalledme=dbstack;
  whocalledme=[whocalledme.name];
  if strfind(whocalledme,'imwrite')  % invoked through imwrite with a 
    % slightly different calling method
    data=varargin{1};
    filename=varargin{3};
  else
    data=varargin{1};
    filename=varargin{2};
    datatype=varargin{3};
  end
else
  disp('Not understood')
  help(mfilename)
end
if ~exist('datatype','var')
  % disp('Datatype being determined from the variable')
  % sets datatype from type of data in matlab
  if isa(data,'double')
    datatype='float64';
  elseif isa(data,'single')
    datatype='float32';
  elseif isa(data,'uint8')
  datatype='uint8';
  elseif isa(data,'uint16')
    datatype='uint16';
  elseif isa(data,'uint32')
    datatype='uint32';
  end
end
size(data);
filename;
datatype;
% now setup ESRF datatype name
switch datatype
	case 'uint8',
	esrfdatatype='UnsignedByte';
	nbytes=1;
	case 'uint16',
	esrfdatatype='UnsignedShort';
	nbytes=2;
	case 'uint32',
	esrfdatatype='UnsignedLong';
	nbytes=4;
	case 'float32',
	esrfdatatype='FloatValue';
	nbytes=4;
	case 'float64',
	esrfdatatype='DoubleValue';
	nbytes=8;
end
nimage=1;


[fid,msg]=fopen(filename,'w','l');
if fid==-1
  fprintf('Could not open %s\n',filename);
  fprintf('%s\n',msg);
  return
end
% header
header=sfGenerateHeader(esrfdatatype,nbytes,size(data));
fwrite(fid,header);
fwrite(fid,permute(data,[2 1 3]),datatype);
fclose(fid);
end

function header=sfGenerateHeader(datatype,nbytes,dimensions)
headerlength=1024;
header=sprintf('{\n');
header=[header headerstring('HeaderID','EH:000001:000000:000000','string')];
header=[header headerstring('Image',1,'integer')];
header=[header headerstring('ByteOrder','LowByteFirst','string')];
header=[header headerstring('DataType',datatype,'string')];
header=[header headerstring('Dim_1',dimensions(2),'integer')];
header=[header headerstring('Dim_2',dimensions(1),'integer')];
if length(dimensions)==3
  header=[header headerstring('Dim_3',dimensions(3),'integer')];
end  
header=[header headerstring('Size',nbytes*prod(dimensions),'integer')];
header=[header headerstring('Date',date,'string')];


header=[header blanks(headerlength-2-length(header)) sprintf('%s\n','}')];

end

